h1. Disclaimer
This report summarize the efficiency of a project with various metrics.

{%- for metric_report in metric_reports %}
h1. {{ metric_report.metric_name }}
{%- for issue_type in metric_report.report_data["issue_types"] %}
h2. Status per {{ issue_type["type"] }}
!{{ issue_type["path"] }}.png|align=center, hspace=0!
{%- endfor %}
{%- endfor %}
